==> fcitx5-0.0.0.20200428-1-aarch64.pkg.tar.xz: usr/include/Fcitx5/Utils/fcitx-utils/dbus/servicewatcher.h <==
//
// Copyright (C) 2016~2016 by CSSlayer
// wengxt@gmail.com
//
// This library is free software; you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 2.1 of the
// License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; see the file COPYING. If not,
// see <http://www.gnu.org/licenses/>.
//
#ifndef _FCITX_UTILS_DBUS_SERVICEWATCHER_H_
#define _FCITX_UTILS_DBUS_SERVICEWATCHER_H_

#include <memory>
#include <string>
#include <fcitx-utils/dbus/bus.h>
#include <fcitx-utils/handlertable.h>
#include <fcitx-utils/macros.h>

namespace fcitx {
namespace dbus {

typedef std::function<void(const std::string &serviceName,
                           const std::string &oldOwner,
                           const std::string &newOwner)>
    ServiceWatcherCallback;
typedef HandlerTableEntry<ServiceWatcherCallback> ServiceWatcherEntry;

class ServiceWatcherPrivate;

class FCITXUTILS_EXPORT ServiceWatcher {
public:
    ServiceWatcher(Bus &bus);
    ~ServiceWatcher();

    // unlike regular NameOwnerChanged signal, this will also initiate a
    // GetNameOwner call to avoid race condition
    // if GetNameOwner returns, it will intiate a call (name, "", owner) if
    // service exists, otherwise (name, "", "")
    FCITX_NODISCARD std::unique_ptr<ServiceWatcherEntry>
    watchService(const std::string &name, ServiceWatcherCallback callback);

private:
    std::unique_ptr<ServiceWatcherPrivate> d_ptr;
    FCITX_DECLARE_PRIVATE(ServiceWatcher);
};
} // namespace dbus
} // namespace fcitx

#endif // _FCITX_UTILS_DBUS_SERVICEWATCHER_H_

==> fcitx5-5.1.7-3-aarch64.pkg.tar.xz: usr/include/Fcitx5/Utils/fcitx-utils/dbus/servicewatcher.h <==
/*
 * SPDX-FileCopyrightText: 2016-2016 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */
#ifndef _FCITX_UTILS_DBUS_SERVICEWATCHER_H_
#define _FCITX_UTILS_DBUS_SERVICEWATCHER_H_

#include <memory>
#include <string>
#include <fcitx-utils/dbus/bus.h>
#include <fcitx-utils/handlertable.h>
#include <fcitx-utils/macros.h>

/// \addtogroup FcitxUtils
/// \{
/// \file
/// \brief API for service monitoring.

namespace fcitx {
namespace dbus {

typedef std::function<void(const std::string &serviceName,
                           const std::string &oldOwner,
                           const std::string &newOwner)>
    ServiceWatcherCallback;
typedef HandlerTableEntry<ServiceWatcherCallback> ServiceWatcherEntry;

class ServiceWatcherPrivate;

/**
 * A class to be used to monitor the services registered on the Bus.
 */
class FCITXUTILS_EXPORT ServiceWatcher {
public:
    ServiceWatcher(Bus &bus);
    ~ServiceWatcher();

    // unlike regular NameOwnerChanged signal, this will also initiate a
    // GetNameOwner call to avoid race condition
    // if GetNameOwner returns, it will intiate a call (name, "", owner) if
    // service exists, otherwise (name, "", "")
    FCITX_NODISCARD std::unique_ptr<ServiceWatcherEntry>
    watchService(const std::string &name, ServiceWatcherCallback callback);

private:
    std::unique_ptr<ServiceWatcherPrivate> d_ptr;
    FCITX_DECLARE_PRIVATE(ServiceWatcher);
};
} // namespace dbus
} // namespace fcitx

#endif // _FCITX_UTILS_DBUS_SERVICEWATCHER_H_
